@@ -327,7 +327,10 @@ class Text_Dataset:
             return
         self.log.info("Encoding text corpora as ngrams.")
         for text in self.text_list:
-            self.log.info(f"Encoding text {text['title']}...")
+            if 'alias' in text:
+                self.log.info(f"Encoding text {text['alias']}...")
+            else:
+                self.log.info(f"Encoding text {text['title']}...")
             text["text_encoded"] = self.encode(text["text"])
         self.log.info("Encoding text corpora as ngrams done.")
 
@@ -778,7 +781,10 @@ class Text_Dataset:
                     if p - 1 > mxQ:
                         mxQ = p - 1
                         mxI = text["index"]
-                        mxN = f"{text['author']}: {text['title']}"
+                        if 'alias' in text:
+                            mxN = f"{text['alias']}"
+                        else:
+                            mxN = f"{text['author']}: {text['title']}"
                         found = True
             if found:  # save longest quote for colorizing
                 if len(noquote) > 0:
